(define-syntax (property stx)
  (syntax-case stx ()
    ((_) (raise-syntax-error #f "property requires at least one name" stx))
    ((_ name)
     (syntax
      (begin
        (define property-name null)
        (define/public name
          (case-lambda
            [() property-name]
            [(value) (set! property-name value)])))))
    ((_ name names ...)
     (syntax
      (begin
        (property name)
        (property names ...))))))

(define-syntax (define-entity stx)
  (syntax-case stx ()
    ((_ name (p ps ...))
     (syntax
      (define name
        (class* object%()
          (property p)
          (property ps ...)
          (super-new)))))))

(define-entity service-call%
  (customer-name customer-id))

(define lines
  '("SVCLGRANT     1024      "
    "SVCLVICKY     1025      "))

(define (parse-svcl-line line)
  (let ([instance (make-object service-call%)]
        [name (substring line 5 14)]
        [id (substring line 14 24)])
    (send instance customer-name name)
    (send instance customer-id id)
    instance))

(define mappings (make-hash-table 'equal))

(hash-table-put! mappings "SVCL" parse-svcl-line)

(define (process-line line)
  (let* ([type (substring line 0 4)]
         [handler (hash-table-get mappings type null)])
    (if (not (null? handler))
        (handler line)
        #f)))

(define results (map process-line lines))