(define-syntax (property stx)
  (syntax-case stx ()
    ((_) (raise-syntax-error #f "property requires at least one name" stx))
    ((_ name)
     (syntax
      (begin
        (define property-name null)
        (define/public name
          (case-lambda
            [() property-name]
            [(value) (set! property-name value)])))))
    ((_ name names ...)
     (syntax
      (begin
        (property name)
        (property names ...))))))

(define-syntax (define-entity stx)
  (syntax-case stx ()
    ((_ name (p ps ...))
     (syntax
      (define name
        (class* object%()
          (property p)
          (property ps ...)
          (super-new)))))))

(define-entity service-call%
  (customer-name customer-id))

(define lines
  '("SVCLGRANT1024"
    "SVCLVICKY1025"
    "INVALID LINE"))

(define (parse-svcl-line line)
  (let ([instance (make-object service-call%)]
        [name (substring line 4 9)]
        [id (substring line 9 13)])
    (send instance customer-name name)
    (send instance customer-id id)
    instance))

(define prefix->type (make-hash-table 'equal))

(define type->populator (make-hash-table 'equal))

(hash-table-put! prefix->type "SVCL" service-call%)

(hash-table-put! type->populator service-call% parse-svcl-line)

(define (process-line line)
  (let* ([prefix (substring line 0 4)]
         [type (hash-table-get prefix->type prefix null)])
    (if (not (null? type))
        (let ([populator (hash-table-get type->populator type null)])
          (if (not (null? populator))
              (populator line)
              #f))
        #f)))

(define results (map process-line lines))

(display (send (car results) customer-name))
{newline}
(display (send (car results) customer-id))