(define-syntax (property stx)
  (syntax-case stx ()
    ((_) (raise-syntax-error #f "property requires at least one name" stx))
    ((_ name)
     (begin
       (if (not (identifier? (syntax name)))
           (raise-syntax-error 
            #f 
            "property names must be identifiers, not expressions" 
            (syntax name)))
       (syntax
        (begin
          (define property-name null)
          (define/public name
            (case-lambda
              [() property-name]
              [(value) (set! property-name value)]))))))
    ((_ name names ...)
     (syntax
      (begin
        (property name)
        (property names ...))))))

(define-syntax (define-entity stx)
  (syntax-case stx ()
    ((_ name (p ps ...))
     (syntax
      (define name
        (class* object%()
          (property p)
          (property ps ...)
          (super-new)))))))

(define-syntax (define-mapping stx)
  (syntax-case stx ()
    ((_ type prefix (start stop field) ...)
     (syntax
      (begin
        (define (populator line)
          (let ([instance (make-object type)])
            (begin
              (send instance field (substring line start stop)) ...
              instance)))
        (hash-table-put! prefix->type prefix type)
        (hash-table-put! type->populator type populator))))))

(define-entity service-call%
  (customer-name customer-id))

(define-entity sales-call%
  (customer-name customer-id))

(define lines
  '("SVCLGRANT1024"
    "SVCLVICKY1025"
    "SLCLHAPPY1026"))

(define prefix->type (make-hash-table 'equal))

(define type->populator (make-hash-table 'equal))

(define-mapping
  service-call%
  "SVCL"
  (4 9 customer-name)
  (9 13 customer-id))

(define-mapping
  sales-call%
  "SLCL"
  (4 9 customer-name)
  (9 13 customer-id))

(define (process-line line)
  (let* ([prefix (substring line 0 4)]
         [type (hash-table-get prefix->type prefix null)])
    (if (not (null? type))
        (let ([populator (hash-table-get type->populator type null)])
          (if (not (null? populator))
              (populator line)
              #f))
        #f)))

(define results (map process-line lines))

(hash-table-for-each prefix->type (λ (key val) 
                                    (printf "Key: ~a, Value: ~a~n" key val)))

(hash-table-for-each type->populator (λ (key val) 
                                       (printf "Key: ~a, Value: ~a~n" key val)))

(send (car results) customer-name)
(send (car results) customer-id)